{% extends "django_tables2/table.html" %}

{% load i18n django_tables2 %}

{% block pagination %}
    {{ block.super }}
    {% block table.report %}
        {% if table.is_configured %}
            {% for label, format in table.formats %}
                <a href="{% querystring table.param_report=format %}">
                    <img src="{{ STATIC_URL }}img/{{ format }}_icon.png" title="{{ label }}"/>
                </a>
            {% endfor %}
        {% endif %}
    {% endblock table.report %}
{% endblock %}